/**************************************************
  CodeVisionAVR C Compiler
  (C) 2003-2005 AVRMALL, by Jong-Hyung Kim

  Prototypes for Graphic LCD 128*64 functions 
  www.avrmall.com 

  [LCD]   [M128 PORT]
   1 GND  -    GND
   2 +5V  -    VCC  
   3 VLC  - LCD contrast control voltage 0..1V
   4 RS   - 41  A14
   5 RD   - 33  WR
   6 EN   -     74HC00 8pin
   7 D0   - 51  PA0
   8 D1   - 50  PA1
   9 D2   - 49  PA2
  10 D3   - 48  PA3
  11 D4   - 47  PA4
  12 D5   - 46  PA5
  13 D6   - 45  PA6
  14 D7   - 44  PA7
  15 CS1  - 15  PB6
  16 CS2  - 16  PB5
  17 RST  - 17  PB7
  18 Vout - 
   
***************************************************************************/

#include <mega128.h>
#include <string.h>
#include <stdio.h>
#include <delay.h>
#include "am_gralcd.h"

#define UDRE            0x20
#define Setbit(x,y)     (x |= (1<<y))
#define Clrbit(x,y)     (x &= (~(1<<y)))
#define Cjkbit(x,y)     (x & (1<<y))

#define SRE             7
#define SRW             6

#define CS1On Setbit(PORTB,6)
#define CS1Off Clrbit(PORTB,6)
#define CS2On Setbit(PORTB,5)
#define CS2Off Clrbit(PORTB,5)
#define ResetOn Setbit(PORTB,7)
#define ResetOff Clrbit(PORTB,7)





void data_write1(char outchar)
{
        char *lcd_io;    
        check_busy1();    
        CS1Off;        
        lcd_io = (char *) 0xc000;
        *lcd_io = outchar;
        CS1On;
}

void data_write2(char outchar)
{
        char *lcd_io;    
        check_busy2();    
        CS2Off;        
        lcd_io = (char *) 0xc000;
        *lcd_io = outchar;
        CS2On;
}

char inst_read1(char outchar)
{
        char *lcd_io;
        CS1Off;        
        lcd_io = (char *) 0x8000;
        outchar = *lcd_io;
        CS1On;        
        return(outchar);  
}

char inst_read2(char outchar)
{
        char *lcd_io;
        CS2Off;        
        lcd_io = (char *) 0x8000;
        outchar = *lcd_io;
        CS2On;        
        return(outchar);  
}

void inst_write1(char outchar)
{
        char *lcd_io;
        check_busy1();    
        CS1Off;    
        lcd_io = (char *) 0x8000;
        *lcd_io = outchar;
        CS1On;    
}  

void inst_write2(char outchar)
{
        char *lcd_io;
        check_busy2();    
        CS2Off;
        lcd_io = (char *) 0x8000;
        *lcd_io = outchar;
        CS2On;
}  

void check_busy1(void)
{
        char lcd_status;
        while(inst_read1(lcd_status) & 0x80);
}

void check_busy2(void)
{
        char lcd_status;
        while(inst_read2(lcd_status) & 0x80);
}

void InitDisplay(void)
{
        CS1On;
        CS2On;
    
        Setbit(MCUCR, SRE);
        Setbit(MCUCR, SRW);
    
        ResetOn;
        delay_us(100);    
        ResetOff;
        delay_us(100);    
        ResetOn;        
    
        check_busy1();    
        inst_write1(0x3f);
        delay_us(100);

        check_busy1();    
        inst_write1(0xc0);
        delay_us(100);

        check_busy2();
        inst_write2(0x3f);
        delay_us(100);

        check_busy2();    
        inst_write2(0xc0);
        delay_us(100);

}

void PixelXY(char x, char y, int ch)
{
        int temp;

        if( y<64 )
        {
                inst_write1(0xb8 + x);  //x page setting
                temp ++;                // used for time delay
                temp ++;
                inst_write1(0x40 + y);  //y address setting
                temp ++;
                temp ++;
                data_write1(ch);
        }

        if( (y>=64) && (y<128) )
        {
                inst_write2(0xb8 + x);  //x page setting
                temp ++;
                temp ++;
                inst_write2(0x40 + (y-64));  //y address setting
                temp ++;
                temp ++;
                data_write2(ch);
        }
}

void ClearLcd(void)
{
        unsigned int i,j;
        for(i=0;i<8;i++)
        {
                for(j=0;j<128;j++)
                PixelXY(i,j,0x00); 
        }
}

void atmelBmpPuts(void)
{
    char i,j;
    
    for(j=0; j<128; j++)
    {                                          //lcd location caution!!
        for(i=0;i<8;i++)                       //lcd(0,0) is right, over point
            PixelXY(7-i,127-j,~atmel_fnt[j*8+i]);   //7-i, 127-j mean change order
    }
}