/***********************************************
Chip type           : ATmega128
Program type        : Application
Clock frequency     : 16.000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 256

By Park Jung Hyun
www.AVRMALL.com   
Used with CodeVisionAVR Ver 1.24.2c Standard.
*********************************************/

// CodeVisionAVR C Compiler
// (C) 1998-2004 Pavel Haiduc, HP InfoTech S.R.L.

// I/O registers definitions for the ATmega128


#pragma used+
sfrb PINF=0;
sfrb PINE=1;
sfrb DDRE=2;
sfrb PORTE=3;
sfrb ADCL=4;
sfrb ADCH=5;
sfrw ADCW=4;      // 16 bit access
sfrb ADCSRA=6;
sfrb ADMUX=7;
sfrb ACSR=8;
sfrb UBRR0L=9;
sfrb UCSR0B=0xa;
sfrb UCSR0A=0xb;
sfrb UDR0=0xc;
sfrb SPCR=0xd;
sfrb SPSR=0xe;
sfrb SPDR=0xf;
sfrb PIND=0x10;
sfrb DDRD=0x11;
sfrb PORTD=0x12;
sfrb PINC=0x13;
sfrb DDRC=0x14;
sfrb PORTC=0x15;
sfrb PINB=0x16;
sfrb DDRB=0x17;
sfrb PORTB=0x18;
sfrb PINA=0x19;
sfrb DDRA=0x1a;
sfrb PORTA=0x1b;
sfrb EECR=0x1c;
sfrb EEDR=0x1d;
sfrb EEARL=0x1e;
sfrb EEARH=0x1f;
sfrw EEAR=0x1e;   // 16 bit access
sfrb SFIOR=0x20;
sfrb WDTCR=0x21;
sfrb OCDR=0x22;
sfrb OCR2=0x23;
sfrb TCNT2=0x24;
sfrb TCCR2=0x25;
sfrb ICR1L=0x26;
sfrb ICR1H=0x27;
sfrw ICR1=0x26;   // 16 bit access
sfrb OCR1BL=0x28;
sfrb OCR1BH=0x29;
sfrw OCR1B=0x28;  // 16 bit access
sfrb OCR1AL=0x2a;
sfrb OCR1AH=0x2b;
sfrw OCR1A=0x2a;  // 16 bit access
sfrb TCNT1L=0x2c;
sfrb TCNT1H=0x2d;
sfrw TCNT1=0x2c;  // 16 bit access
sfrb TCCR1B=0x2e;
sfrb TCCR1A=0x2f;
sfrb ASSR=0x30;
sfrb OCR0=0x31;
sfrb TCNT0=0x32;
sfrb TCCR0=0x33;
sfrb MCUCSR=0x34;
sfrb MCUCR=0x35;
sfrb TIFR=0x36;
sfrb TIMSK=0x37;
sfrb EIFR=0x38;
sfrb EIMSK=0x39;
sfrb EICRB=0x3a;
sfrb RAMPZ=0x3b;
sfrb XDIV=0x3c;
sfrb SPL=0x3d;
sfrb SPH=0x3e;
sfrb SREG=0x3f;
#pragma used-


// Interrupt vectors definitions


// CodeVisionAVR C Compiler
// (C) 1998-2000 Pavel Haiduc, HP InfoTech S.R.L.


#pragma used+

void delay_us(unsigned int n);
void delay_ms(unsigned int n);

#pragma used-

/**************************************************
  CodeVisionAVR C Compiler
  (C) 1998-2000 AVRMall, by Park Jung Hyun

  Prototypes for LCD functions 
***************************************************/



#pragma used+

void lcd_init(void);
void lcd_data(char data);
void lcd_cmd(char cmd);
char lcd_read(char read);
void lcd_clear(void);
void lcd_display_on(void);
void lcd_display_off(void);
void lcd_right_shift(void);
void lcd_left_shift(void);
void lcd_busy(void);
void lcd_ch(char ch);
void lcd_str(char *str);
void Gotoxy(unsigned char x, unsigned char y);    

void _lcd_str(flash char *str);

#pragma used-

/**************************************************
  CodeVisionAVR C Compiler
  (C) 1998-2000 AVRMall, by Park Jung Hyun
  LCD functions
  
  www.avrmall.com 
***************************************************/
  
// CodeVisionAVR C Compiler
// (C) 1998-2004 Pavel Haiduc, HP InfoTech S.R.L.

// I/O registers definitions for the ATmega128






// CodeVisionAVR C Compiler
// (C) 1998-2000 Pavel Haiduc, HP InfoTech S.R.L.





//#include <mem.h>

void lcd_init(void)
void lcd_init(void)
void lcd_init(void)
void lcd_init(void)
{
	
	delay_ms(15);
	lcd_cmd(0x38);
	
	delay_ms(5);
	lcd_cmd(0x38);
	
	delay_us(200);
	lcd_cmd(0x38);
	delay_us(100);
	
	lcd_cmd(0x38);
	delay_us(100);
	
	lcd_display_on();

	lcd_clear();
	
	lcd_cmd(0x06);
	lcd_busy();
}

void lcd_data(char data)
{
	char *lcd_io;
	lcd_busy();	
	//pokeb(0xc000, data);	//writes the byte data to SRAM at 0xc000
	//pokeb(0x9002, data);	//writes the byte data to SRAM at 0xc000
	//delay_us(10);
	lcd_io = (char *) 0x9002; 
        *lcd_io = data;
        //delay_us(10);
}

void lcd_cmd(char cmd)
{
	char *lcd_io;
	lcd_busy();	
	//pokeb(0x8000, cmd);	//writes the byte data to SRAM at 0x8000
	//pokeb(0x9000, cmd);	//writes the byte data to SRAM at 0x8000
	//delay_us(10);
	lcd_io = (char *) 0x9000; 
        *lcd_io = cmd;
        //delay_us(10);
}

char lcd_read(char read)
{	
	char *lcd_io;
	//read=peekb(0x8000); //reads a byte located in SRAM at 0x8000
	//read=peekb(0x9001); //reads a byte located in SRAM at 0x8000  
	//delay_us(10);
	lcd_io = (char *) 0x9001;
        read = *lcd_io;  
        //delay_us(10);      	
	return(read);
}
void lcd_clear(void)
{
	lcd_busy();
        lcd_cmd(0x01);
        delay_us(100);
}

void lcd_display_on(void)
{
	lcd_busy();
	lcd_cmd(0x0c);
        delay_us(100);
}

void lcd_display_off(void)
{
	lcd_busy();
	lcd_cmd(0x08);
        delay_us(100);
}

void lcd_right_shift(void)
{
	lcd_busy();
        lcd_cmd(0x1c);
        delay_us(100);
}

void lcd_left_shift(void)
{
	lcd_busy();
        lcd_cmd(0x18);
        delay_us(100);
}

void lcd_busy(void)
{
	char lcd_status;
	while(lcd_read(lcd_status) & 0x80);
}

void lcd_ch(char ch)            
{
    	char *lcd_io;
    	lcd_busy();
    	//pokeb(0xc000, ch);
    	lcd_io = (char *) 0x9002; 
        *lcd_io = ch;
}

void lcd_str(char *str)         
{
 	unsigned char i = 0;
 	do{
		lcd_ch(str[i]) ;
   	}while(str[++i] != 0x00);	//null=0x00
}

void _lcd_str(flash char *str)
{
	unsigned char i = 0;
 	do{
		lcd_ch(str[i]) ;
   	}while(str[++i] != 0x00);	//null=0x00
}

void Gotoxy(unsigned char x, unsigned char y)	//x-  y |
{
    lcd_busy();
    switch(y)
    {
       case 0: lcd_cmd(0x80+x); break;  
       case 1: lcd_cmd(0xc0+x); break;
       case 2: lcd_cmd(0x94+x); break;
       case 3: lcd_cmd(0xd4+x); break;	
    }
	
}
//#include <lcd.h>

void main(void)
{
        ////////////////////////lcd out string
        //char lcd_buff[]="::AVRMall::";
        //char lcd_buff_1[]="Mega128 LCD Test";
        //////////////////////////////////////
        
        MCUCR=0xc0;
	#asm("sei");
	
	lcd_init();
	
	while(1)
	{
		Gotoxy(1,0);
		_lcd_str("** AVRMALL **");
	
		Gotoxy(0,1);
		_lcd_str("M128KIT V2.0 LCD"); 
		/*	
		lcd_display_on();
		delay_ms(300);
		lcd_display_off();
		delay_ms(300);
		lcd_display_on();
		delay_ms(300);
	        	
		delay_ms(500);delay_ms(500);      
	        	
		lcd_right_shift();
		lcd_right_shift();
		delay_ms(500);delay_ms(500);
	        	
		lcd_left_shift();
		lcd_left_shift(); 
		lcd_left_shift();
		lcd_left_shift();
		delay_ms(500);
	               	
		lcd_right_shift();
		lcd_right_shift();    
                */
	}
}
